#include "IADSREnvelope.h"


IADSREnvelope::IADSREnvelope(IPlugBase* pPlugBase, IPlugNrpn* pPlugNrpn, IGraphics* pGraphics,  IRECT rect, int numberOfTab, int index, int* attackIds, int* nrpnAttackParams) 
	: IMultipleControlContainer(pPlugBase, pPlugNrpn, rect, numberOfTab, NUMBER_OF_POINTS)
{
	char buf[15];
	insideRect.L = rect.L + 3;
	insideRect.R = rect.R - 3;
	insideRect.T = rect.T + 3;
	insideRect.B = rect.B - 3;

	sizeDiv4 = (insideRect.R - insideRect.L) / 4;
	height = insideRect.B - insideRect.T;
	
	drag = -1;
	this->attackIds = new int[numberOfTab];
	this->nrpnAttackParams = new int[numberOfTab];

	for (int k=0; k<numberOfTab; k++) {
		this->attackIds[k] = attackIds[k];
		this->nrpnAttackParams[k] = nrpnAttackParams[k];
	}


	for (int k=0; k<numberOfTab; k++) {
		sprintf(buf, "E%i Attack", k+1 + index);
		mPlug->GetParam(attackIds[k] + ATTACK)->InitInt(buf, 20, 0, 255, "");
		sprintf(buf, "E%i Decay", k+1 + index);
		mPlug->GetParam(attackIds[k] + DECAY)->InitInt(buf, 60, 0, 255, "");
		sprintf(buf, "E%i Sustion", k+1 + index);
  		mPlug->GetParam(attackIds[k] + SUSTAIN)->InitInt(buf, 200, 0, 255, "");
		sprintf(buf, "E%i Release", k+1 + index);
  		mPlug->GetParam(attackIds[k] + RELEASE)->InitInt(buf, 40, 0, 255, "");

		singleControls[k][ATTACK] = new ISingleControl(k*4 + ATTACK, pPlugBase, this, attackIds[k] + ATTACK);
		pGraphics->AttachControl(singleControls[k][ATTACK]);
		singleControls[k][DECAY] = new ISingleControl(k*4 + DECAY, pPlugBase, this, attackIds[k] + DECAY);
		pGraphics->AttachControl(singleControls[k][DECAY]);
		singleControls[k][SUSTAIN] = new ISingleControl(k*4 + SUSTAIN, pPlugBase, this, attackIds[k] + SUSTAIN);
		pGraphics->AttachControl(singleControls[k][SUSTAIN]);
		singleControls[k][RELEASE] = new ISingleControl(k*4 + RELEASE, pPlugBase, this, attackIds[k] + RELEASE);
		pGraphics->AttachControl(singleControls[k][RELEASE]);
	}	
	
	computeRect();
}


IADSREnvelope::~IADSREnvelope(void)
{
}

void IADSREnvelope::computeRect() {
	int xAttack = insideRect.L + sizeDiv4 * currentValues[currentTab][ATTACK] / 255;
	int xDecay = xAttack + sizeDiv4 * currentValues[currentTab][DECAY] / 255;
	int ySustain = insideRect.B - height * currentValues[currentTab][SUSTAIN] / 255;
	int xRelease = xDecay + sizeDiv4 + sizeDiv4 * currentValues[currentTab][RELEASE] / 255;

	rect[ATTACK].L = xAttack - 3 ;
	rect[ATTACK].R = xAttack + 3 ;
	rect[ATTACK].T = insideRect.T - 3;
	rect[ATTACK].B = insideRect.T + 3 ;

	rect[DECAY].L = xDecay - 3 ;
	rect[DECAY].R = xDecay + 3;
	rect[DECAY].T = ySustain - 3;
	rect[DECAY].B = ySustain + 3;

	rect[SUSTAIN].L = xDecay + sizeDiv4 - 3;
	rect[SUSTAIN].R = xDecay + sizeDiv4 + 3;
	rect[SUSTAIN].T = ySustain - 3;
	rect[SUSTAIN].B = ySustain + 3;

	rect[RELEASE].L = xRelease - 3;
	rect[RELEASE].R = xRelease + 3;
	rect[RELEASE].T = insideRect.B - 3;
	rect[RELEASE].B = insideRect.B + 3;
}


bool IADSREnvelope::Draw(IGraphics* pGraphics) {
	const char* labels [] = { "Attk", "Deca", "Sust", "Rele" };
	float x1, y1, x2, y2;

	x2 = (rect[0].L + rect[0].R)/2.0f;
	y2 = (rect[0].T + rect[0].B)/2.0f;
	pGraphics->DrawLine(&white, (float)insideRect.L, (float)insideRect.B, x2, y2, (const IChannelBlend*) NULL, true);
	
	for (int k=1; k<NUMBER_OF_POINTS; k ++) {
		x1 = (rect[k-1].L + rect[k-1].R)/2.0f;
		y1 = (rect[k-1].T + rect[k-1].B)/2.0f;
		x2 = (rect[k].L + rect[k].R)/2.0f;
		y2 = (rect[k].T + rect[k].B)/2.0f;
		pGraphics->DrawLine(&white, x1, y1, x2, y2, (const IChannelBlend*) NULL, true);
	}

	for (int k=0; k<NUMBER_OF_POINTS; k ++) {
		if (drag == k) {
			x1 = (rect[k].L + rect[k].R)/2.0f;
			y1 = (rect[k].T + rect[k].B)/2.0f;
			pGraphics->DrawLine(&darkYellow, (float)insideRect.L, y1, (float)insideRect.R, y1);
			pGraphics->DrawLine(&darkYellow, x1,(float) insideRect.T, x1, (float)insideRect.B);
			pGraphics->FillIRect(&yellow, &rect[k]);
		} else {
			pGraphics->FillIRect(&yellow, &rect[k]);
		//	pGraphics->DrawRect(&yellow, &rect[k]);
		}
	}

	IText itext(12, &yellow);
	for (int k=0; k<NUMBER_OF_POINTS; k ++) {
		char buffer [10];
		sprintf (buffer, "%s: %i", labels[k], currentValues[currentTab][k]);
		IRECT textREct = IRECT(insideRect.L + sizeDiv4*3 + 10, insideRect.T + k*16 , insideRect.R, insideRect.T + k*16 + 16);
		pGraphics->DrawIText(&itext, buffer, &textREct);
	}
		
	return true;
}



void IADSREnvelope::OnMouseDown(int x, int y, IMouseMod* pMod) {
	for (int k=0; k<NUMBER_OF_POINTS; k ++) {
		if (x>=rect[k].L && x<=rect[k].R && y<=rect[k].B && y>=rect[k].T) {
			drag = k;
			SetDirty(-1);
		}
	}
}

void IADSREnvelope::OnMouseDrag(int x, int y, int dX, int dY, IMouseMod* pMod) {
	if (drag == -1) {
		return;
	}

	IMidiMsg pMsg;
	int oldValue = currentValues[currentTab][drag];
	switch (drag) {
		case ATTACK:
			currentValues[currentTab][ATTACK] += dX;
			break;
		case DECAY:
			currentValues[currentTab][DECAY] += dX;
			break;
		case SUSTAIN:
			currentValues[currentTab][SUSTAIN] -= dY;
			break;
		case RELEASE:
			currentValues[currentTab][RELEASE] += dX;
			break;
	}
	if (currentValues[currentTab][drag]<0) {
		currentValues[currentTab][drag] = 0;
	} else if (currentValues[currentTab][drag]>255) {
		currentValues[currentTab][drag] = 255;
	}
	if (currentValues[currentTab][drag]!=oldValue) {
		computeRect();
		SetDirty(drag);
	    Redraw();
	}
}

void IADSREnvelope::OnMouseUp(int x, int y, IMouseMod* pMod) {
	if (drag != -1) {
		SetDirty(drag);
		drag = -1;
	}
}


bool IADSREnvelope::setCurrentTab(int newTab) {
	if (ISingleControlContainerBase::setCurrentTab(newTab)) {
		computeRect();
		return true;
	}
	return false;
}



void IADSREnvelope::SetDirty(int whichValue) {
	IMultipleControlContainer::SetDirty(whichValue);
	computeRect();
}


void IADSREnvelope::setValueFromHardware(int tab, int controlNumber, int nrpnValue) {
	IMultipleControlContainer::setValueFromHardware(tab, controlNumber, nrpnValue);
	computeRect();
}


int IADSREnvelope::getNrpnParam(int currentTab, int controlNumber) {
	return nrpnAttackParams[currentTab] + controlNumber;;
}
int IADSREnvelope::getParamId(int currentTab, int controlNumber) {
	return attackIds[currentTab] + controlNumber;
}
float IADSREnvelope::getFloatFromNrpn(int nrpnValue) {
	return (float)nrpnValue / 255.0f;
}
float IADSREnvelope::getFloatFromInternal(int intValue) {
	return (float)intValue / 255.0f;
}
int   IADSREnvelope::getInternalFromFloat(float value) {
	return (int) (value * 255.0f);
}
int   IADSREnvelope::getNrpnFromInternal(int nrpnValue) {
	return nrpnValue;
}



void IADSREnvelope::flushNrpn() {
	for (int tab=0; tab<numberOfTab; tab++) {
		for (int cn=0; cn<numberOfControls; cn++) {
			nrpnPlug->sendNrpn(getNrpnParam(tab, cn), getNrpnFromInternal(currentValues[tab][cn]) );
		}
	}
}
