#include "IControlSwitch.h"


IControlSwitch::IControlSwitch(IPlugBase* pPlugBase, IPlugNrpn* pPlugNrpn, IGraphics* pGraphics, IRECT rect,const char* name, int* paramIds, int numberOfTab, int numberOfControl, const char**controlNames, ISingleControlContainerBase**controlsToSwitch)
	:  ISingleControlContainer(pPlugBase, pPlugNrpn, rect, numberOfTab,  paramIds, NULL)
{
	char buf[30];
	this-> numberOfControls = numberOfControl;

	for (int k=0; k<numberOfTab; k++) {
		singleControls[k] = new ISingleControl(k, pPlugBase, this, paramIds[k]);
		pGraphics->AttachControl(singleControls[k]);

		sprintf(buf, name, k+1);
		mPlug->GetParam(paramIds[k])->InitInt(buf, 0, 0, numberOfControl-1, "");

		this->paramIds[k] = paramIds[k];
		this->currentValues[k] = 0;
	}

	
	this->controlNames = new char*[numberOfControl];
	this->controlsToSwitch = new  ISingleControlContainerBase*[numberOfControl];
	for (int k=0; k<numberOfControl; k++) {
		this->controlNames[k] = new char[30];
		strcpy(this->controlNames[k], controlNames[k]);
		if (controlsToSwitch != NULL && controlsToSwitch[k] != NULL) {
			this->controlsToSwitch[k] = controlsToSwitch[k];
		} else {
			this->controlsToSwitch[k] = NULL;
		}
	}

	this->controlsToSwitch[1]->setDisable();
	
}



IControlSwitch::~IControlSwitch(void)
{
}


bool IControlSwitch::Draw(IGraphics* pGraphics) {
	IRECT textRect1 = IRECT(mRECT.L, mRECT.T, mRECT.R, mRECT.T+12);
	int sizeX = mRECT.R - mRECT.L;
	int sizeY = (mRECT.B - mRECT.T) / 3;
	int sizeButton = sizeX/4;
	IText itext1(12, &yellow);
	IText itext2(12, &gray);
	IRECT drawRECT = IRECT(mRECT.L, mRECT.T + sizeY, mRECT.R, mRECT.B - sizeY);
	IRECT textRect2 = IRECT(mRECT.L, mRECT.B-12, mRECT.R, mRECT.B);
	
	if (this->currentValues[this->currentTab] == 0) {		
		pGraphics->DrawIText(&itext1, this->controlNames[0], &textRect1);
	} else {
		pGraphics->DrawIText(&itext2, this->controlNames[0], &textRect1);
	}

	pGraphics->DrawRect(&white, &drawRECT);
	
	IRECT buttonRECT;	
	if (this->currentValues[this->currentTab] == 1) {		
		pGraphics->DrawIText(&itext1, this->controlNames[1], &textRect2);
		buttonRECT = IRECT(drawRECT.R-sizeButton, drawRECT.T+2, drawRECT.R-1, drawRECT.B-1);
	} else {
		pGraphics->DrawIText(&itext2, this->controlNames[1], &textRect2);
		buttonRECT = IRECT(drawRECT.L+2, drawRECT.T+2, drawRECT.L+1+sizeButton, drawRECT.B-1);
	}
	pGraphics->FillIRect(&yellow, &buttonRECT);

	return true;
}

void IControlSwitch::OnMouseDown(int x, int y, IMouseMod* pMod) {
	if (this->currentValues[this->currentTab] == 0) {
		this->currentValues[this->currentTab] = 1;
	} else {
		this->currentValues[this->currentTab] = 0;
	}
	updateControlVisibility();
	SetDirty(true);
}


void IControlSwitch::updateControlVisibility() {
	if (this->currentValues[this->currentTab] == 1) {
		this->controlsToSwitch[0]->setDisable();
		this->controlsToSwitch[1]->setEnable();
	} else {
		this->controlsToSwitch[0]->setEnable();
		this->controlsToSwitch[1]->setDisable();
	}
}

bool IControlSwitch::setCurrentTab(int newTab) {
	if (ISingleControlContainer::setCurrentTab(newTab)) {
		updateControlVisibility();
		return true;
	}
	return false;
}

void IControlSwitch::setValueFromHardware(int tab, int controlNumber, int nrpnValue) {
	ISingleControlContainer::setValueFromHardware(tab, controlNumber, nrpnValue);
	updateControlVisibility();
}


// implement abstract methods
float IControlSwitch::getFloatFromNrpn(int nrpnValue)  {
	return (float)nrpnValue;
}

float IControlSwitch::getFloatFromInternal(int intValue) {
	return (float)intValue;
}

int   IControlSwitch::getInternalFromFloat(float value) {
	return (int) value >.5 ? 1 : 0;
}

int   IControlSwitch::getNrpnFromInternal(int intValue) {
	return intValue;
}

