
#include "math.h"
#include "IKnob.h"


#define _PI 3.141592653589793238f



IKnob::IKnob(IPlugBase* pPlugBase,IPlugNrpn* pPlugNrpn, IGraphics* pGraphics, IRECT rect, int numberOfTab, const char*vstName, int* paramIds, int* nrpnParams, int nrpnMinValue, int nrpnMaxValue, int nrpnDefaultValue, DisplayType displayType, int* accurateNumber, int numberOfStrings, const char** strings )
	 :  ISingleControlContainer(pPlugBase, pPlugNrpn, rect, numberOfTab,  paramIds, nrpnParams)
{
	char buf[20];

	// nrpnValues must have pBitmap->N values
	this->nrpnMaxValue = nrpnMaxValue;
	this->nrpnDefaultValue = nrpnDefaultValue;
	this->nrpnMinValue = nrpnMinValue;
	this->displayType = displayType;

	for (int k=0; k<numberOfTab; k++) {
		this->currentValues[k] = nrpnDefaultValue;
		this->singleControls[k] = new ISingleControl(k, pPlugBase, this, paramIds[k]);

		pGraphics->AttachControl(singleControls[k]);
		if (accurateNumber == NULL) {
			sprintf(buf,vstName, k+1);
		} else {
			sprintf(buf,vstName, accurateNumber[k]);
		}
		mPlug->GetParam(paramIds[k])->InitInt(buf, nrpnDefaultValue, nrpnMinValue, nrpnMaxValue, "");
	}

	// initialize once the following float
	cx = (mRECT.L + mRECT.R) /2.0f; 
	cy = (mRECT.T + mRECT.B) /2.0f; 
	r = (MIN(mRECT.B - mRECT.T, mRECT.R - mRECT.L))/2.0f;

	this->numberOfStrings = numberOfStrings;
	this->strings = new char*[numberOfStrings];
	for (int k=0; k< numberOfStrings; k++) {
		this->strings[k] = new char[20];
		strcpy(this->strings[k], strings[k]);
	}
}


bool IKnob::Draw(IGraphics* pGraphics) {
	static float start = _PI + _PI/8.0f;
	static float end = start + 14.0f/8.0f*_PI;

	IText itext(12, &yellow);
	IText itextgray(12, &gray);

	IRECT textREct = IRECT(mRECT.L, (int)cy -6, mRECT.R, (int)cy+6);

	IColor white = ISingleControlContainer::white;
	IColor yellow = ISingleControlContainer::yellow;

	if (mGrayed) {
		white = ISingleControlContainer::gray;
		yellow = ISingleControlContainer::gray;
		itext.mColor = yellow;
	}


	float angle = start + 14.0f/8.0f*_PI * (currentValues[currentTab] - nrpnMinValue) / (nrpnMaxValue + numberOfStrings - nrpnMinValue);
	pGraphics->DrawArc(&yellow, cx, cy, r-5, start, angle, NULL, true);
	pGraphics->DrawArc(&white, cx, cy, r-2, start, end, NULL, true);
	pGraphics->DrawArc(&white, cx, cy, r-8, start, end, NULL, true);

	char display[10];
	if (currentValues[currentTab] <= (this->nrpnMaxValue - this->numberOfStrings)) {
		switch (this->displayType) {
		case DISPLAY_FLOAT_3_5:
			sprintf(display, "%1.2f", (float)currentValues[currentTab] / 32);
			break;
		case DISPLAY_FLOAT_4_4:
			sprintf(display, "%1.2f", (float)currentValues[currentTab] / 16);
			break;
		case DISPLAY_FLOAT_5_3:
			sprintf(display, "%1.2f", (float)currentValues[currentTab] / 8);
			break;
		case DISPLAY_FLOAT_1_7:
			sprintf(display, "%1.2f", (float)currentValues[currentTab] / 128);
			break;
		case DISPLAY_CHAR:		
		case DISPLAY_UCHAR:
			sprintf(display, "%i", currentValues[currentTab]);
			break;
		default:
			sprintf(display, "BURK");
		}
		if (this->numberOfStrings >0) {
			textREct.L -= 40;
			textREct.R -= 40;
		}
		pGraphics->DrawIText(&itext, display, &textREct);
		if (this->numberOfStrings >0) {
			textREct.L += 85;
			textREct.R += 85;
			strcpy(display, this->strings[0]);
			pGraphics->DrawIText(&itextgray, display, &textREct);
		}
	} else {
		textREct.L += 45;
		textREct.R += 45;
		int index = currentValues[currentTab] - (this->nrpnMaxValue - this->numberOfStrings) - 1;
		strcpy(display, this->strings[index]);
		pGraphics->DrawIText(&itext, display, &textREct);

		textREct.L -= 85;
		textREct.R -= 85;
		switch (this->displayType) {
		case DISPLAY_FLOAT_4_4:
			sprintf(display, "%1.2f", (float)(this->nrpnMaxValue - this->numberOfStrings) / 16);
			break;
		case DISPLAY_UCHAR:
			sprintf(display, "%i", (this->nrpnMaxValue - this->numberOfStrings));
			break;
		default:
			sprintf(display, "BURK");
		}
		pGraphics->DrawIText(&itextgray, display, &textREct);
	}

	return true;
}


void IKnob::OnMouseWheel(int x, int y, IMouseMod* pMod, int d) {
	int oldValue = currentValues[currentTab];
	if (d<0 && currentValues[currentTab]>this->nrpnMinValue) {
			currentValues[currentTab] --;
	}
	if (d>0 && currentValues[currentTab]< nrpnMaxValue) {
			currentValues[currentTab] ++;
	}
	if (oldValue != currentValues[currentTab]) {
		SetDirty(true);
	}
}


void IKnob::OnMouseDown(int x, int y, IMouseMod* pMod) {

	draging = true;
	dragX = x;
	dragY = y;
	dragValue = currentValues[currentTab];
}

void IKnob::OnMouseDblClick(int x, int y, IMouseMod* pMod) {
	if (this->currentValues[currentTab] != this->nrpnDefaultValue) {
		this->currentValues[currentTab] = this->nrpnDefaultValue;
		SetDirty(true);		
	}
}

void IKnob::OnMouseDrag(int x, int y, int dX, int dY, IMouseMod* pMod) {
	if (!draging) {
		return;
	}

	int oldValue = currentValues[currentTab];


	float addValue;
	if (pMod->S && !slowDrag) {
		currentValues[currentTab] = (int) (dragValue + (dragY - y)  / 255.0f * nrpnMaxValue);
		dragValue = currentValues[currentTab];
		slowDrag= true;
		dragY = y;
		dragX = x;
	}  else if (!pMod->S && slowDrag) {
		currentValues[currentTab] = (int) (dragValue + (dragY - y)  / 2550.0f * nrpnMaxValue);
		dragValue = currentValues[currentTab];
		slowDrag= false;
		dragY = y;
		dragX = x;
	}


	if (slowDrag) {
		addValue = (dragY - y)  / 2550.0f * nrpnMaxValue;
	} else {
		addValue = (dragY - y)  / 255.0f * nrpnMaxValue;
	}
	currentValues[currentTab] = dragValue + (int)addValue;

	if (currentValues[currentTab]<this->nrpnMinValue) {
		currentValues[currentTab] = this->nrpnMinValue;
	} else if (currentValues[currentTab]>nrpnMaxValue) {
		currentValues[currentTab] = nrpnMaxValue;
	}

	if (currentValues[currentTab] !=oldValue) {
		SetDirty(true);
	    Redraw();
	}
}

void IKnob::OnMouseUp(int x, int y, IMouseMod* pMod) {
	draging = false;
}


// implement abstract methods
float IKnob::getFloatFromNrpn(int nrpnValue)  {
	return (float)nrpnValue  / (this->nrpnMaxValue -  this->nrpnMinValue);
}

float IKnob::getFloatFromInternal(int intValue) {
	return ((float)intValue - this->nrpnMinValue) / (this->nrpnMaxValue - this->nrpnMinValue);
}

int IKnob::getInternalFromFloat(float value) {
	return (int) ((value + .0005f) * ((double)this->nrpnMaxValue - this->nrpnMinValue)) + (double)this->nrpnMinValue;
}

int IKnob::getNrpnFromInternal(int intValue) {
	return intValue - this->nrpnMinValue;
}

