#include <string.h>
#include "IOverlaySelect.h"

// paramIds : vstParams index
// x, y: top corner
// pBitmap :



IOverlaySelect::IOverlaySelect(IPlugBase* pPlugBase,IPlugNrpn* pPlugNrpn, IGraphics* pGraphics,  IRECT rect, int numberOfTabs, const char*vstName, int* paramIds, int* nrpnParams, int numberOfValues, int* nrpnValues, const char**valueLabels, int* accurateNumber )
	 : ISingleControlContainer(pPlugBase, pPlugNrpn, rect, numberOfTabs, paramIds, nrpnParams)
{
	char buf[20];


	this->numberOfValues = numberOfValues;
	if (this->numberOfValues > 40) {
		this->cellSizeY = 14; 
	} else {
		this->cellSizeY = 18; 
	}
	this->valueLabels = new char*[numberOfValues];
	for (int k=0; k<numberOfValues; k++) {
		this->valueLabels[k] = new char[30];
		strcpy(this->valueLabels[k], valueLabels[k]);
	}

	int sizeY = this->cellSizeY * numberOfValues;
	int middleY = ( mRECT.T + mRECT.B) / 2;
	this->fullRECT = IRECT(mRECT.L, middleY-sizeY/2, mRECT.R + 80, middleY+sizeY / 2);
	if(this->fullRECT.T<0) {
		int decal = -this->fullRECT.T;
		this->fullRECT.T = 0;
		this->fullRECT.B += decal;
	}
	int decalX = -1;
	if(this->fullRECT.R>900) {
		decalX = this->fullRECT.R-900;
		this->fullRECT.L -= decalX;
		this->fullRECT.R -= decalX;
	}

	this->overlayRECT = IRECT(mRECT.R, fullRECT.T, fullRECT.R, fullRECT.B);
	if (decalX != -1) {
		this->overlayRECT.L -= decalX;
	}
	this->valueRECT = mRECT;

	// nrpnValues must have numberOfValues values
	this->nrpnValues = new int[numberOfValues];
	for (int k=0; k<numberOfValues; k++) {
		this->nrpnValues[k] = nrpnValues[k];
	}

	singleControls = new ISingleControl*[numberOfTabs];
	for (int k=0; k<numberOfTabs; k++) {
		singleControls[k] = new ISingleControl(k, pPlugBase, this, paramIds[k]);
		pGraphics->AttachControl(singleControls[k]);
		if (accurateNumber == NULL) {
			sprintf(buf, vstName, k+1);
		} else {
			sprintf(buf, vstName, accurateNumber[k]);
		}
		mPlug->GetParam(paramIds[k])->InitInt(buf, 0, 0, numberOfValues-1, "");
	}
	overlayed = false;
}


bool IOverlaySelect::Draw(IGraphics* pGraphics) {
	IColor white = ISingleControlContainer::white;
	IColor yellow = ISingleControlContainer::yellow;

	if (mGrayed) {
		white = ISingleControlContainer::gray;
		yellow = ISingleControlContainer::gray;
	}

	if (!overlayed) {
		mRECT = valueRECT;
		SetTargetArea(valueRECT);
	}
	
	if (!overlayed) {
		IText itext(12, &yellow);
		pGraphics->DrawRect(&white, &valueRECT);
		int textDY = (valueRECT.B - valueRECT.T) / 2 - 6;
		pGraphics->DrawIText(&itext, this->valueLabels[this->currentValues[this->currentTab]], &IRECT(valueRECT.L, valueRECT.T+ textDY, valueRECT.R, valueRECT.B));
	} else  {
		IText itext(12, &yellow);
		IText itext2(11, &white);
		IText itext3(11, &yellow);
		int textDY = (valueRECT.B - valueRECT.T) / 2 - 6;
		int sizeY = this->cellSizeY;

		pGraphics->FillIRect(&shadow, &valueRECT);
		pGraphics->DrawRect(&white, &valueRECT);
		pGraphics->DrawIText(&itext, this->valueLabels[this->currentValues[this->currentTab]], &IRECT(valueRECT.L, valueRECT.T+ textDY, valueRECT.R, valueRECT.B));
		pGraphics->FillIRect(&shadow, &overlayRECT);
		pGraphics->DrawRect(&gray, &IRECT(overlayRECT.L, overlayRECT.T, overlayRECT.R -1, overlayRECT.B-1));

		for (int k=0; k<this->numberOfValues; k++)  {
			IRECT iRECT(overlayRECT.L, overlayRECT.T + k*sizeY, overlayRECT.R -1, overlayRECT.T + (k+1)*sizeY -1);
			pGraphics->DrawLine(&gray, (float)iRECT.L, (float)iRECT.B, (float)iRECT.R, (float)iRECT.B);	
			int textY =  iRECT.T+ 3;
			if (this->numberOfValues > 40) {
				int textY =  iRECT.T ;
			}
			IRECT iRECTText(iRECT.L, textY, iRECT.R, iRECT.B);
			if (k != this->currentValues[currentTab]) {
				pGraphics->DrawIText(&itext2, this->valueLabels[k], &iRECTText);
			} else {
				pGraphics->DrawIText(&itext3, this->valueLabels[k], &iRECTText);
			}
		}
	}
	return true;
}


void IOverlaySelect::OnMouseDown(int x, int y, IMouseMod* pMod) {
	if (valueRECT.Contains(x,y) && !overlayed) {
		overlayed = true;
		mRECT = fullRECT;
		SetTargetArea(fullRECT);
		SetDirty(false);
		return;
	}  
	if (this->overlayed && overlayRECT.Contains(x,y) && !pMod->R) {
		this->currentValues[currentTab] = (y - overlayRECT.T) / this->cellSizeY;
	}
	if (this->overlayed) {
		overlayed=false;
		SetDirty(true);
	}
}

void IOverlaySelect::OnMouseWheel(int x, int y, IMouseMod* pMod, int d) {
	if (d<0 && this->currentValues[currentTab]< numberOfValues-1) {
		this->currentValues[currentTab] ++;
		SetDirty(true);
	} else if (d>0 && this->currentValues[currentTab]>0) {
		this->currentValues[currentTab] --;
		SetDirty(true);
	}
}

// implement abstract methods
float IOverlaySelect::getFloatFromNrpn(int nrpnValue)  {
	int realValue = -1;
	for (int k=0; k<numberOfValues && realValue == -1; k++) {
		if (nrpnValues[k] == nrpnValue) {
			realValue = k;
		}
	}
	if (realValue != -1) {
		return (float)realValue /  (numberOfValues -1);
	}
	return 0.0f ;
}

float IOverlaySelect::getFloatFromInternal(int intValue) {
	return (float)intValue /  (numberOfValues -1);
}

int   IOverlaySelect::getInternalFromFloat(float value) {
	return (int)( value * (numberOfValues -1));
}

int   IOverlaySelect::getNrpnFromInternal(int intValue) {
	return nrpnValues[intValue] ;
}
