#include "IPNGSelect.h"

// paramIds : vstParams index
// x, y: top corner
// pBitmap :


IPNGSelect::IPNGSelect(IPlugBase* pPlugBase,IPlugNrpn* pPlugNrpn, IGraphics* pGraphics, int* paramIds, int x, int y, IBitmap* pBitmap, int numberOfTab, int* nrpnParams, int* nrpnValues, int spaceX )
	 : ISingleControlContainer(pPlugBase, pPlugNrpn, x, y, pBitmap, numberOfTab, paramIds, nrpnParams)
{
	char buf[20];

	this->numberOfValues = pBitmap->N;
	this->mRECT.L = x;
	this->mRECT.R = x + pBitmap->W  + 22 + spaceX*2;
	this->mRECT.T = y;
	this->mRECT.B = y + pBitmap->H / pBitmap->N;
	SetTargetArea(mRECT);

	// nrpnValues must have pBitmap->N values
	this->nrpnValues = new int[pBitmap->N];
	for (int k=0; k<pBitmap->N; k++) {
		this->nrpnValues[k] = nrpnValues[k];
	}

	this->spaceX = spaceX;
	singleControls = new ISingleControl*[numberOfTab];
	for (int k=0; k<numberOfTab; k++) {
		singleControls[k] = new ISingleControl(k, pPlugBase, this, paramIds[k]);
		pGraphics->AttachControl(singleControls[k]);

		sprintf(buf, "Osc%i Shape", k+1);
		mPlug->GetParam(paramIds[k])->InitInt(buf, 0, 0, pBitmap->N -1, "");

		this->paramIds[k] = paramIds[k];
		this->nrpnParams[k] = nrpnParams[k];
		this->currentValues[k] = 0;
	}
}


bool IPNGSelect::Draw(IGraphics* pGraphics) {
	pGraphics->DrawLine(&white, mRECT.L+11.0f, mRECT.T+10.0f, mRECT.L+11.0f, mRECT.B-10.0f,  (const IChannelBlend*) NULL, true);
	pGraphics->DrawLine(&white, mRECT.L+11.0f, mRECT.T+10.0f, mRECT.L+0.0f, (mRECT.B + mRECT.T)/2.0f,  (const IChannelBlend*) NULL, true);
	pGraphics->DrawLine(&white, mRECT.L+11.0f, mRECT.B-10.0f, mRECT.L+0.0f, (mRECT.B + mRECT.T)/2.0f,  (const IChannelBlend*) NULL, true);

	pGraphics->DrawLine(&white, mRECT.R-11.0f , mRECT.T+10.0f, mRECT.R-11.0f , mRECT.B-10.0f,  (const IChannelBlend*) NULL, true);
	pGraphics->DrawLine(&white, mRECT.R-11.0f , mRECT.T+10.0f, mRECT.R+0.0f , (mRECT.B + mRECT.T)/2.0f,  (const IChannelBlend*) NULL, true);
	pGraphics->DrawLine(&white, mRECT.R-11.0f , mRECT.B-10.0f, mRECT.R +0.0f, (mRECT.B + mRECT.T)/2.0f,  (const IChannelBlend*) NULL, true);

	IRECT bmRECT = IRECT(mRECT.L+11+ spaceX, mRECT.T, mRECT.R-11 - spaceX, mRECT.B);
    pGraphics->DrawBitmap(&mBitmap, &bmRECT , currentValues[currentTab] +1, &mBlend);
	if (spaceX>0) {
		pGraphics->DrawRect(&white, &bmRECT );
	}
	return true;
}

void IPNGSelect::OnMouseDown(int x, int y, IMouseMod* pMod) {
	IRECT leftRect(mRECT.L, mRECT.T, (mRECT.R + mRECT.L)/2 -1, mRECT.B);
	IRECT rightRect((mRECT.R + mRECT.L)/2 +1, mRECT.T, mRECT.R, mRECT.B);
	int oldValue = currentValues[currentTab];

	if (leftRect.Contains(x, y)) {
		currentValues[currentTab] --;
	} else if (rightRect.Contains(x, y)) {
		currentValues[currentTab] ++;
	}
	
	if (currentValues[currentTab] < 0) {
		currentValues[currentTab] = numberOfValues-1 ;
	}
	if (currentValues[currentTab] > numberOfValues-1) {
		currentValues[currentTab] = 0;
	}
	if (oldValue != currentValues[currentTab]) {
		SetDirty(true);
		Redraw();
	}
}

/*

void IPNGSelect::SetValueFromHardware(int tab, int controlNumber, double value)
{
	int intValue = value * numberOfValues;
	int realValue = -1;
	for (int k=0; k<numberOfValues; k++) {
		if (nrpnValues[k] == intValue) {
			realValue = k;
		}
	}
	if (realValue != -1) {
		singleControls[tab]->SetValueFromUserInput(realValue);
		currentValues[tab] = realValue;
		if (tab == currentTab) {
			mPlug->SetParameterFromGUI(paramIds[currentTab], realValue /  (numberOfValues -1));
			mDirty = true;
			Redraw();
		}
	}
}

*/


// implement abstract methods
float IPNGSelect::getFloatFromNrpn(int nrpnValue)  {
	int realValue = -1;
	for (int k=0; k<numberOfValues; k++) {
		if (nrpnValues[k] == nrpnValue) {
			realValue = k;
		}
	}
	if (realValue != -1) {
		return (float)realValue /  (numberOfValues -1);
	}
	return 0.0f ;
}

float IPNGSelect::getFloatFromInternal(int intValue) {
	return (float)intValue /  (numberOfValues -1);
}

int   IPNGSelect::getInternalFromFloat(float value) {
	return (int) ((value) * (numberOfValues -1));
}

int   IPNGSelect::getNrpnFromInternal(int intValue) {
	return nrpnValues[intValue] ;
}
