
#include "ISingleControlContainer.h"


ISingleControlContainer::ISingleControlContainer(IPlugBase* pPlugBase, IPlugNrpn* pPlugNrpn, IRECT rect, int numberOfTab,  int* paramIds, int* nrpnParams) 
	: ISingleControlContainerBase(pPlugBase, pPlugNrpn, rect, numberOfTab)
{
	// paramIds must have numberOfTab
	this->paramIds = new int[numberOfTab];
	// currentValues must have numberOfTab
	this->currentValues = new int[numberOfTab];
	// nrpnParams must have numberOfTab
	if (nrpnParams != NULL) {
		this->nrpnParams = new int[numberOfTab];
	} else {
		this->nrpnParams = NULL;
	}

	this->numberOfTab = numberOfTab;
	this->currentTab = 0;

	this->singleControls = new ISingleControl*[numberOfTab];

	for (int k=0; k<numberOfTab; k++) {
		this->paramIds[k] = paramIds[k];
		if (nrpnParams!=NULL) {
			this->nrpnParams[k] = nrpnParams[k];
		}
	}
}


ISingleControlContainer::ISingleControlContainer(IPlugBase* pPlugBase, IPlugNrpn* pPlugNrpn,  int x, int y, IBitmap* pBitmap, int numberOfTab,  int* paramIds, int* nrpnParams) 
	: ISingleControlContainerBase( pPlugBase, pPlugNrpn, x, y, pBitmap, numberOfTab) {
	// paramIds must have numberOfTab
	this->paramIds = new int[numberOfTab];
	// currentValues must have numberOfTab
	this->currentValues = new int[numberOfTab];
	if (nrpnParams != NULL) {
		this->nrpnParams = new int[numberOfTab];
	} else {
		this->nrpnParams = NULL;
	}

	this->numberOfTab = numberOfTab;
	this->currentTab = 0;

	this->singleControls = new ISingleControl*[numberOfTab];

	for (int k=0; k<numberOfTab; k++) {
		this->currentValues[k] = 0;
		this->paramIds[k] = paramIds[k];
		if (nrpnParams!=NULL) {
			this->nrpnParams[k] = nrpnParams[k];
		}
	}
}

void ISingleControlContainer::flushNrpn() {
	for (int k=0; k<numberOfTab; k++) {
		nrpnPlug->sendNrpn(this->nrpnParams[k], getNrpnFromInternal(currentValues[k]));
	}
}



void ISingleControlContainer::setValueFromHardware(int tab, int controlNumber, int nrpnValue)
{	
	float floatValue = getFloatFromNrpn(nrpnValue);
	// control Number is equal to tab
	// singleControls[tab]->SetValueFromUserInput(floatValue);
	currentValues[tab] = getInternalFromFloat(floatValue);
	mPlug->SetParameterFromGUI(paramIds[tab], floatValue);
	if (tab == currentTab) {
		mDirty = true;
	    Redraw();
	}
}

void ISingleControlContainer::setValueFromAutomation(int number, double value) {
	int intValue = getInternalFromFloat(value);
	if (currentValues[number] != intValue || this->nrpnPlug->isInitializing()) {
		currentValues[number] = intValue;
		if (nrpnParams!=NULL) {
			nrpnPlug->sendNrpn(nrpnParams[number], getNrpnFromInternal(intValue));	
			if (number  == currentTab) {
				SetDirty(false);
			}
		}
	} 
}

void ISingleControlContainer::SetDirty(bool pushParamToPlug) {
	mDirty = true;
	Redraw();
	if (pushParamToPlug) {
		double normalizedValue =  getFloatFromInternal(currentValues[currentTab]);
		// singleControls[currentTab]->SetValueFromUserInput(normalizedValue);
		mPlug->SetParameterFromGUI(paramIds[currentTab], normalizedValue);
		if (this->nrpnParams != NULL) {
			nrpnPlug->sendNrpn(nrpnParams[currentTab], getNrpnFromInternal(currentValues[currentTab]));
		}
	}
}


