#include "IStepSequencer.h"


IStepSequencer::IStepSequencer(IPlugBase* pPlugBase,IPlugNrpn* pPlugNrpn, IGraphics* pGraphics,  IRECT rect, int firstParam, int numberOfTab) 
	: IMultipleControlContainer(pPlugBase, pPlugNrpn, rect, numberOfTab, NUMBER_OF_STEPS)
{
	char buf[20];

	this->drag = -1;
	this->firstParam = firstParam;	
	this->sizeStepX = (mRECT.R - mRECT.L) / NUMBER_OF_STEPS;
	this->sizeStepY = (mRECT.B - mRECT.T) -4;

	for (int k=0; k<numberOfTab; k++) {
		for (int j=0; j<NUMBER_OF_STEPS; j++) {
			int index = j + k*NUMBER_OF_STEPS;
			sprintf(buf, "Step %i-%i", k, j);
			mPlug->GetParam(firstParam + k*NUMBER_OF_STEPS+ j)->InitInt(buf, 0, 0, 15, "");
			this->singleControls[k][j] = new ISingleControl(index, pPlugBase, this, firstParam + index);
			pGraphics->AttachControl(singleControls[k][j]);
		}	
	}
}


IStepSequencer::~IStepSequencer(void)
{
}



bool IStepSequencer::Draw(IGraphics* pGraphics) {
	
	for (int k=0; k<NUMBER_OF_STEPS; k++) {
		IRECT mRECT1 = IRECT(mRECT.L + this->sizeStepX*k + 2, mRECT.T, mRECT.L + this->sizeStepX*k + this->sizeStepX -2, mRECT.B);
		if ((k % 4) == 0) {
			pGraphics->DrawRect(&white, &mRECT1);
		} else {
			pGraphics->DrawRect(&gray, &mRECT1);
		}
		int top = (int)(mRECT.B - 1.0f - this->sizeStepY*currentValues[currentTab][k]/15.0f );
		int bottom = mRECT.B-1;
		IRECT mRECT2 = IRECT(mRECT.L + this->sizeStepX*k + 4, top, mRECT.L + this->sizeStepX*k + this->sizeStepX -3, bottom);
		pGraphics->FillIRect(&yellow, &mRECT2);
		if (drag == k) {
			pGraphics->DrawRect(&white, &IRECT(mRECT1.L+1, mRECT1.T+1, mRECT1.R-1, mRECT1.B-1));
		}
	}

	IText itext(12, &white);	
	for (int k=0; k<NUMBER_OF_STEPS; k ++) {
		char code[2];
		if (currentValues[currentTab][k]<10) {
			code[0] = '0' + currentValues[currentTab][k];
		} else {
			code[0] = 'A' + currentValues[currentTab][k] - 10;
		}
		code[1] = 0;
		IRECT mRECT2 = IRECT(mRECT.L + this->sizeStepX*k + 4,mRECT.B -16, mRECT.L + this->sizeStepX*k + this->sizeStepX -3, mRECT.B);
		pGraphics->DrawIText(&itext, code, &mRECT2);
	}

	return true;
}


void IStepSequencer::OnMouseDown(int x, int y, IMouseMod* pMod) {
	drag = (int) (((float)x - mRECT.L) * NUMBER_OF_STEPS / (mRECT.R - mRECT.L)) ;
	if (pMod->R) {
		currentValues[currentTab][drag] = 0;
	} else {
		currentValues[currentTab][drag] = (int)(((float)mRECT.B - y) / ((float) mRECT.B - mRECT.T) * 15.0f);
	}
	SetDirty(drag);
}


void IStepSequencer::OnMouseDblClick(int x, int y, IMouseMod* pMod) {
	drag = (int) (((float)x - mRECT.L) * NUMBER_OF_STEPS / (mRECT.R - mRECT.L)) ;
	currentValues[currentTab][drag] = 15;
	SetDirty(drag);
}

void IStepSequencer::OnMouseDrag(int x, int y, int dX, int dY, IMouseMod* pMod) {
	int oldValue = currentValues[currentTab][drag];
	drag = (int) (((float)x - mRECT.L) * NUMBER_OF_STEPS / (mRECT.R - mRECT.L)) ;
	if (drag>15) {
		drag = 15;
	} else if (drag<0) {
		drag = 0;
	}
	if (pMod->R) {
		currentValues[currentTab][drag] = 0;
	} else {
		currentValues[currentTab][drag] = (int)(((float)mRECT.B - y) / ((float) mRECT.B - mRECT.T) * 15.0f);
	}
	if (currentValues[currentTab][drag]>15) {
		currentValues[currentTab][drag] = 15;
	} 
	if (currentValues[currentTab][drag]<0) {
		currentValues[currentTab][drag] = 0;
	} 
	SetDirty(drag);
}

void IStepSequencer::OnMouseUp(int x, int y, IMouseMod* pMod) {
	if (drag != -1) {
		SetDirty(-1);
		drag = -1;
	}
}

void IStepSequencer::OnMouseWheel(int x, int y, IMouseMod* pMod, int d) {
	drag = (int) (((float)x - mRECT.L) * NUMBER_OF_STEPS / (mRECT.R - mRECT.L)) ;
	if (drag>15) {
		drag = 15;
	} else if (drag<0) {
		drag = 0;
	}
	if (d>0 && currentValues[currentTab][drag]<15) {
		currentValues[currentTab][drag]++;
	} 
	if (d<0 && currentValues[currentTab][drag]>0) {
		currentValues[currentTab][drag]--;
	} 

	SetDirty(drag);
}

int IStepSequencer::getNrpnParam(int currentTab, int controlNumber) {
	return	2 * 128 + currentTab * 128 + controlNumber;;
}
int IStepSequencer::getParamId(int currentTab, int controlNumber) {
	return firstParam + currentTab*NUMBER_OF_STEPS+ controlNumber;
}
float IStepSequencer::getFloatFromNrpn(int nrpnValue) {
	return (float)nrpnValue / 15.0f;
}
float IStepSequencer::getFloatFromInternal(int intValue) {
	return (float)intValue / 15.0f;
}
int   IStepSequencer::getInternalFromFloat(float value) {
	return (int) (value * 15.0f);
}
int   IStepSequencer::getNrpnFromInternal(int nrpnValue) {
	return nrpnValue;
}
