#include "ITabSelect.h"


ITabSelect::ITabSelect(IPlugBase* pPlugBase,IPlugNrpn* pPlugNrpn, IRECT rect, const char* title, int numberOfTabs) : IControl(pPlugBase, rect)
{
	this->title = title;
	this->numberOfTabs = numberOfTabs; 
	this->currentTab = 0; 
}


ITabSelect::~ITabSelect(void)
{
}


bool ITabSelect::Draw(IGraphics* pGraphics) {
	IColor darker(100, 0, 0, 0);
	IColor blue(255,102,130,174);
	IColor white(255,255,255,255);
	IColor yellow(255,240,180,30);

	int tabSize = (mRECT.R - mRECT.L) / numberOfTabs-2;
	for (int k=0; k<numberOfTabs; k++) {
		IRECT rect(mRECT.L+k*tabSize, mRECT.T, mRECT.L+((k+1)*tabSize)-1, mRECT.B-1);
		pGraphics->FillIRect(&darker, &rect);
		if (k == currentTab) {
			pGraphics->DrawRect(&white, &rect);
		}
	}

	IText itext(12, &white);
	IText itextSel(12, &yellow);
	int yPos = (mRECT.T + mRECT.B)/2 - 7; 
	for (int k=0; k<numberOfTabs; k ++) {
		char buffer [20];
		sprintf (buffer, "%s  %i", title, k+1);
		IRECT rect(mRECT.L+k*tabSize, yPos, mRECT.L+((k+1)*tabSize), yPos+14);
		if (k == currentTab) {
			pGraphics->DrawIText(&itextSel, buffer, &rect);
		} else {
			pGraphics->DrawIText(&itext, buffer, &rect);
		}
	}

	return true;
}

void ITabSelect::OnMouseDown(int x, int y, IMouseMod* pMod) {
	int tabSize = (mRECT.R - mRECT.L) / numberOfTabs;
	for (int k=0; k<numberOfTabs; k++) {
		IRECT rect(mRECT.L+k*tabSize, mRECT.T, mRECT.L+((k+1)*tabSize), mRECT.B);
		if (rect.Contains(x, y)) {
			currentTab = k;
			SetDirty(false);
			tabChanged(currentTab);
		}
	}
}


void ITabSelect::addControl(ISingleControlContainerBase *control) {
	controls.Add(control);
}


void ITabSelect::tabChanged(int newTab) {
	int i, n = controls.GetSize();
	ISingleControlContainerBase** ppControl = controls.GetList();
	for (i = 0; i < n; ++i, ++ppControl) {
		(*ppControl) -> setCurrentTab(newTab);		
	}
}
