#ifndef __IPLUGEFFECT__
#define __IPLUGEFFECT__

#include "IPlug_include_in_plug_hdr.h"
#include "IPlugNrpn.h"

#include "ISingleControlContainerBase.h"

class IPresetNameControl;

struct Nrpn {
    unsigned char paramLSB;
    unsigned char  paramMSB;
    unsigned char  valueLSB;
    unsigned char  valueMSB;
    bool readyToSend;
};


class PreenFM : public IPlugNrpn, public IPlug
{
public:

  PreenFM(IPlugInstanceInfo instanceInfo);
  ~PreenFM();

  void Reset();
  void OnParamChange(int paramIdx);

  void ProcessDoubleReplacing(double** inputs, double** outputs, int nFrames);

  void ProcessMidiMsg(IMidiMsg* pMsg);
  void flushNrpn(const Nrpn* nrpn );
  Nrpn* findNrpn(Nrpn *p);

  // IPlugNrpn
  bool sendNrpn(int paramNumber, int value) ;
  bool isInitializing() { return isInit; }
  void startInit()  { isInit = true; }
  void endInit()  { isInit = false; }
  const char* getPreenPresetName();

  void PresetsChangedByHost();

  void flushPresetToNrpn();

  bool SerializeState(ByteChunk* pChunk);
  int UnserializeState(ByteChunk* pChunk, int startPos);


private:
	bool isInit;

	// The controls
	ISingleControlContainerBase *opEnvs, *algoSelect;
	ISingleControlContainerBase *knobVelocity, *knobVoices, *knobGlide;
	ISingleControlContainerBase *imKnob[4], *mixKnob[4];
	ISingleControlContainerBase *oscShapes, *oscKeybOrFixed;
	ISingleControlContainerBase *stepSequencer,* oscFreq, *oscFine, *stepBPM, *stepGate, *lfoFreq, *lfoBias, *lfoKSyn;
	ISingleControlContainerBase* matrixMul[3], *matrixSource[3], *matrixDestination[3];
	ISingleControlContainerBase *lfoWav;
	ISingleControlContainerBase *lfoEnv;
	ITextControl *debug;
	ITextControl *version;
	IPresetNameControl *preset;

	struct Nrpn currentNrpn;
	char presetName[13];

	WDL_PtrList<Nrpn> nrpnList;

};

class IGetPresetButton : public IControl
{
public:
    IGetPresetButton(IPlugBase* plugBase, PreenFM* nrpnPlug, IRECT pR, const char * label);

    virtual void OnMouseDown(int x, int y, IMouseMod* pMod);
    void OnMouseUp(int x, int y, IMouseMod* pMod);
    bool Draw(IGraphics* pGraphics);

protected:
	PreenFM* preenFMPlug;
	bool pushed;
	char mylabel[20];
};


class IWritePresetButton : public IGetPresetButton
{
public:
    IWritePresetButton(IPlugBase* plugBase, PreenFM* nrpnPlug, IRECT pR, const char * label);

    virtual void OnMouseDown(int x, int y, IMouseMod* pMod);

public:
};


#endif
